@@ -1,33 +1,48 @@
+import sys
+import time
 import requests
 from jinja2 import Template
 
 
-API = '/api/python'
+API = 'api/python'
 NEWJOB = '{0}/createItem'
 JOB_URL = '{0}/job/{1}'
 DELETE = '{0}/job/{1}/doDelete'
 BUILD = '{0}/job/{1}/build'
 CONFIG = '{0}/job/{1}/config.xml'
-JOBINFO = '{0}/job/{1}' + API
-LIST = '{0}' + API
-LAST_SUCCESS = '{0}/job/{1}/lastSuccessfulBuild' + API
-TEST_REPORT = '{0}/job/{1}/lastSuccessfulBuild/testReport' + API
-LAST_BUILD = '{0}/job/{1}/lastBuild' + API
-LAST_REPORT = '{0}/job/{1}/lastBuild/testReport' + API
+JOBINFO = '{0}/job/{1}/' + API
+LIST = '{0}/' + API
+LAST_SUCCESS = '{0}/job/{1}/lastSuccessfulBuild/' + API
+TEST_REPORT = '{0}/job/{1}/lastSuccessfulBuild/testReport/' + API
+LAST_BUILD = '{0}/job/{1}/lastBuild/' + API
+LAST_REPORT = '{0}/job/{1}/lastBuild/testReport/' + API
 ENABLE = '{0}/job/{1}/enable'
 DISABLE = '{0}/job/{1}/disable'
 
 
-class HttpNotFoundError(Exception):
+class HttpStatusError(Exception):
     pass
 
 
+class HttpNotFoundError(HttpStatusError):
+    pass
+
+
+HTTP_ERROR_MAP = {
+    404: HttpNotFoundError,
+}
+
+
 def _validate(response):
     """
     Verify the status code of the response and raise exception on 404
     """
-    if response.status_code == 404:
-        raise HttpNotFoundError()
+    message = 'HTTP Status: {0}'.format(response.status_code)
+    if response.status_code >= 400:
+        print(message)
+        exception_cls = HTTP_ERROR_MAP.get(response.status_code,
+                                           HttpStatusError)
+        raise exception_cls(message)
     return response
 
 
@@ -122,7 +137,7 @@ class Jenkins(object):
 
     def set_config_xml(self, jobname, config):
         """
-        Update the ``config.xml`` of a existing job.
+        Replace the ``config.xml`` of an existing job.
         """
         return requests.post(self._build_url(CONFIG, jobname),
                              data=config,
@@ -187,11 +202,20 @@ class Jenkins(object):
         return requests.post(self._build_url(NEWJOB), params=params,
                              auth=self.auth)
 
-    def build(self, jobname):
+    def build(self, jobname, wait=False, grace=10):
         """
         Trigger Jenkins to build a job.
+
+        :param wait:
+            If ``True``, wait until job completes building before returning
         """
-        return self._post(BUILD, jobname)
+        response = self._post(BUILD, jobname)
+        if not wait:
+            return response
+        else:
+            time.sleep(grace)
+            self.wait_for_build(jobname)
+            return self.last_result(jobname)
 
     def delete(self, jobname):
         """
@@ -210,3 +234,19 @@ class Jenkins(object):
         Trigger Jenkins to disable a job.
         """
         return self._post(DISABLE, jobname)
+
+    def is_building(self, jobname):
+        """
+        Check if a job is building
+        """
+        return self.last_result(jobname).get('building', True)
+
+    def wait_for_build(self, jobname, poll_interval=3):
+        """
+        Wait until job has finished building
+        """
+        while (self.is_building(jobname)):
+            time.sleep(poll_interval)
+            sys.stdout.write('.')
+            sys.stdout.flush()
+        print('')
